@@ -159,7 +159,7 @@ void HeaderGenerator::writeFunction(QTextStream& s, const AbstractMetaFunction*
         s << func->originalName() << '(';
         QStringList args;
         foreach (const AbstractMetaArgument* arg, func->arguments()) {
-            QString argName = arg->argumentName();
+            QString argName = arg->name();
             const TypeEntry* enumTypeEntry = 0;
             if (arg->type()->isFlags())
                 enumTypeEntry = reinterpret_cast<const FlagsTypeEntry*>(arg->type()->typeEntry())->originator();
@@ -246,6 +246,7 @@ void HeaderGenerator::writeTypeConverterDecl(QTextStream& s, const TypeEntry* ty
         s << INDENT << "static " << type->name() << " toCpp(PyObject* pyobj);" << endl;
         s << INDENT << "static bool isConvertible(PyObject* pyobj);" << endl;
         if (hasCustomConversion) {
+            s << INDENT << "static bool checkType(PyObject* pyobj);" << endl;
             s << INDENT << "static inline PyObject* toPython(void* cppObj) { return toPython(*reinterpret_cast<"
               << type->name() << (isAbstractOrObjectType ? "" : "*") << " >(cppObj)); }" << endl;
             s << INDENT << "static PyObject* toPython(const " << type->name() << "& cppObj);" << endl;
@@ -389,6 +390,8 @@ void HeaderGenerator::finishGeneration()
     s << "#include <basewrapper.h>" << endl;
     s << "#include <bindingmanager.h>" << endl;
     s << "#include <memory>" << endl << endl;
+    if (usePySideExtensions())
+        s << "#include <qsignal.h>" << endl;
     writeExportMacros(s);
 
     QStringList requiredTargetImports = TypeDatabase::instance()->requiredTargetImports();
@@ -423,6 +426,25 @@ void HeaderGenerator::finishGeneration()
 
     s << "// PyType functions, to get the PyObjectType for a type T\n";
     s << sbkTypeFunctions << endl;
+
+    if (usePySideExtensions()) {
+        foreach (AbstractMetaClass* metaClass, classes()) {
+            if (!metaClass->isQObject() || !metaClass->typeEntry()->generateCode())
+                continue;
+
+            s << "template<>" << endl;
+            s << "inline PyObject* createWrapper<" << metaClass->qualifiedCppName() << " >(const ";
+            s << metaClass->qualifiedCppName() << "* cppobj, bool hasOwnership, bool isExactType)" << endl;
+            s << '{' << endl;
+            s << INDENT << "PyObject* pyObj = Shiboken::SbkBaseWrapper_New(reinterpret_cast<SbkBaseWrapperType*>(SbkType<" << metaClass->qualifiedCppName() << " >()),"
+            << "const_cast<" << metaClass->qualifiedCppName() << "*>(cppobj), hasOwnership, isExactType);" << endl;
+            s << INDENT << "PySide::signalUpdateSource(pyObj);" << endl;
+            s << INDENT << "return pyObj;" << endl;
+            s << '}' << endl;
+
+        }
+    }
+
     s << "// Generated converters declarations ----------------------------------" << endl << endl;
     s << convertersDecl;
     s << "} // namespace Shiboken" << endl << endl;
