@@ -47,6 +47,9 @@ namespace Shiboken
 
 static void SbkBaseWrapperType_dealloc(PyObject* pyObj);
 static PyObject* SbkBaseWrapperType_TpNew(PyTypeObject* metatype, PyObject* args, PyObject* kwds);
+static std::list<PyObject*> splitPyObject(PyObject* pyObj);
+static void incRefPyObject(PyObject* pyObj);
+static void decRefPyObjectlist(const std::list<PyObject*> &pyObj);
 
 extern "C"
 {
@@ -121,7 +124,7 @@ SbkBaseWrapperType SbkBaseWrapper_Type = { { {
     /*tp_name*/             "Shiboken.BaseWrapper",
     /*tp_basicsize*/        sizeof(SbkBaseWrapper),
     /*tp_itemsize*/         0,
-    /*tp_dealloc*/          0,
+    /*tp_dealloc*/          deallocWrapperWithPrivateDtor,
     /*tp_print*/            0,
     /*tp_getattr*/          0,
     /*tp_setattr*/          0,
@@ -286,7 +289,7 @@ void walkThroughClassHierarchy(PyTypeObject* currentType, HierarchyVisitor* visi
             continue;
         } else {
             SbkBaseWrapperType* sbkType = reinterpret_cast<SbkBaseWrapperType*>(type);
-            if (sbkType->is_multicpp)
+            if (sbkType->is_user_type)
                 walkThroughClassHierarchy(type, visitor);
             else
                 visitor->visit(sbkType);
@@ -340,7 +343,7 @@ bool setCppPointer(SbkBaseWrapper* wrapper, PyTypeObject* desiredType, void* cpt
 
 bool cppObjectIsInvalid(PyObject* wrapper)
 {
-    if (wrapper == Py_None
+    if (!wrapper || wrapper == Py_None
         || wrapper->ob_type->ob_type != &Shiboken::SbkBaseWrapperType_Type
         || ((Shiboken::SbkBaseWrapper*)wrapper)->validCppObject) {
         return false;
@@ -374,27 +377,44 @@ void deallocWrapperWithPrivateDtor(PyObject* self)
     Py_TYPE(reinterpret_cast<SbkBaseWrapper*>(self))->tp_free(self);
 }
 
-void keepReference(SbkBaseWrapper* self, const char* key, PyObject* referredObject)
+void keepReference(SbkBaseWrapper* self, const char* key, PyObject* referredObject, bool append)
 {
+
+    bool isNone = (!referredObject || (referredObject == Py_None));
+
     if (!self->referredObjects)
-        return;
+        self->referredObjects = new Shiboken::RefCountMap;
+
     RefCountMap& refCountMap = *(self->referredObjects);
-    Py_INCREF(referredObject);
+    if (!isNone)
+        incRefPyObject(referredObject);
+
     RefCountMap::iterator iter = refCountMap.find(key);
-    if (iter != refCountMap.end())
-        Py_DECREF(iter->second);
-    refCountMap[key] = referredObject;
+    if (!append && (iter != refCountMap.end())) {
+        decRefPyObjectlist(iter->second);
+        refCountMap.erase(iter);
+    }
+
+    if (!isNone) {
+        std::list<PyObject*> values = splitPyObject(referredObject);
+        if (append && (iter != refCountMap.end()))
+            refCountMap[key].insert(refCountMap[key].end(), values.begin(), values.end());
+        else
+            refCountMap[key] = values;
+    }
 }
 
 void clearReferences(SbkBaseWrapper* self)
 {
     if (!self->referredObjects)
         return;
+
     RefCountMap& refCountMap = *(self->referredObjects);
     RefCountMap::iterator iter;
     for (iter = refCountMap.begin(); iter != refCountMap.end(); ++iter)
-        Py_DECREF(iter->second);
+        decRefPyObjectlist(iter->second);
     delete self->referredObjects;
+    self->referredObjects = 0;
 }
 
 bool importModule(const char* moduleName, PyTypeObject*** cppApiPtr)
@@ -454,6 +474,9 @@ static void deallocPythonTypes(PyObject* pyObj)
 
 void deallocWrapper(PyObject* pyObj)
 {
+    if (Py_TYPE(pyObj)->tp_del)
+        Py_TYPE(pyObj)->tp_del(pyObj);
+
     SbkBaseWrapper* sbkObj = reinterpret_cast<SbkBaseWrapper*>(pyObj);
     if (sbkObj->weakreflist)
         PyObject_ClearWeakRefs(pyObj);
@@ -469,9 +492,11 @@ void deallocWrapper(PyObject* pyObj)
         destroyParentInfo(sbkObj);
     clearReferences(sbkObj);
 
+    Py_XDECREF(sbkObj->ob_dict);
     delete[] sbkObj->cptr;
     sbkObj->cptr = 0;
     Py_TYPE(pyObj)->tp_free(pyObj);
+
 }
 
 void SbkBaseWrapperType_dealloc(PyObject* pyObj)
@@ -517,21 +542,20 @@ PyObject* SbkBaseWrapperType_TpNew(PyTypeObject* metatype, PyObject* args, PyObj
         newType->cpp_dtor = 0;
         newType->is_multicpp = 1;
     }
+    newType->original_name = "";
     newType->user_data = 0;
     newType->d_func = 0;
     newType->is_user_type = 1;
     return reinterpret_cast<PyObject*>(newType);
 }
 
-
-
-
 void initShiboken()
 {
     static bool shibokenAlreadInitialised = false;
     if (shibokenAlreadInitialised)
         return;
 
+    initTypeResolver();
 #ifdef WITH_THREAD
     PyEval_InitThreads();
 #endif
@@ -601,6 +625,70 @@ void TypeDiscovery::addTypeDiscoveryFunction(Shiboken::TypeDiscoveryFunc func)
     m_discoveryFunctions.push_back(func);
 }
 
+class FindBaseTypeVisitor : public HierarchyVisitor
+{
+    public:
+        FindBaseTypeVisitor(PyTypeObject* typeToFind) : m_found(false), m_typeToFind(typeToFind) {}
+        virtual void visit(SbkBaseWrapperType* node)
+        {
+            if (reinterpret_cast<PyTypeObject*>(node) == m_typeToFind) {
+                m_found = true;
+                finish();
+            }
+        }
+        bool found() const { return m_found; }
+
+    private:
+        bool m_found;
+        PyTypeObject* m_typeToFind;
+};
+
+bool canCallConstructor(PyTypeObject* myType, PyTypeObject* ctorType)
+{
+    FindBaseTypeVisitor visitor(ctorType);
+    walkThroughClassHierarchy(myType, &visitor);
+    if (!visitor.found()) {
+        PyErr_Format(PyExc_TypeError, "%s isn't a direct base class of %s", ctorType->tp_name, myType->tp_name);
+        return false;
+    }
+    return true;
+}
+
+static std::list<PyObject*> splitPyObject(PyObject* pyObj)
+{
+    std::list<PyObject*> result;
+    if (PySequence_Check(pyObj)) {
+        AutoDecRef lst(PySequence_Fast(pyObj, "Invalid keep reference object."));
+        for(int i = 0, i_max = PySequence_Fast_GET_SIZE(lst.object()); i < i_max; i++) {
+            result.push_back(PySequence_Fast_GET_ITEM(lst.object(), i));
+        }
+    } else {
+        result.push_back(pyObj);
+    }
+    return result;
+}
+
+static void incRefPyObject(PyObject* pyObj)
+{
+    if (PySequence_Check(pyObj)) {
+        for(int i = 0, i_max = PySequence_Size(pyObj); i < i_max; i++) {
+            PySequence_GetItem(pyObj, i);
+        }
+    } else {
+        Py_INCREF(pyObj);
+    }
+}
+
+static void decRefPyObjectlist(const std::list<PyObject*> &lst)
+{
+    std::list<PyObject*>::const_iterator iter = lst.begin();
+    while(iter != lst.end()) {
+        Py_DECREF(*iter);
+        iter++;
+    }
+}
+
+
 } // namespace Shiboken
 
 
