@@ -43,6 +43,19 @@ namespace Shiboken
 
 typedef google::dense_hash_map<const void*, PyObject*> WrapperMap;
 
+
+#ifndef NDEBUG
+static void showWrapperMap(const WrapperMap& wrapperMap)
+{
+    printf("-------------------------------\n");
+    printf("WrapperMap: %p (size: %d)\n", &wrapperMap, (int) wrapperMap.size());
+    WrapperMap::const_iterator iter;
+    for (iter = wrapperMap.begin(); iter != wrapperMap.end(); ++iter)
+        printf("key: %p, value: %p (%s)\n", iter->first, iter->second, iter->second->ob_type->tp_name);
+    printf("-------------------------------\n");
+}
+#endif
+
 struct BindingManager::BindingManagerPrivate {
     WrapperMap wrapperMapper;
     void releaseWrapper(void* cptr);
@@ -75,6 +88,18 @@ BindingManager::BindingManager()
 
 BindingManager::~BindingManager()
 {
+#ifndef NDEBUG
+    showWrapperMap(m_d->wrapperMapper);
+#endif
+    assert(m_d->wrapperMapper.size() == 0);
+
+    /* Cleanup hanging references. We just invalidate them as when
+     * the BindingManager is being destroyed the interpreter is alredy
+     * shutting down. */
+    WrapperMap::iterator iter = m_d->wrapperMapper.begin();
+    for (;iter != m_d->wrapperMapper.end(); ++iter)
+        invalidateWrapper(iter->second);
+
     delete m_d;
 }
 
