@@ -59,21 +59,16 @@ static void deinitTypeResolver()
     typeResolverMap.clear();
 }
 
+void Shiboken::initTypeResolver()
+{
+    assert(typeResolverMap.empty());
+    typeResolverMap.set_empty_key("");
+    typeResolverMap.set_deleted_key("?");
+    std::atexit(deinitTypeResolver);
+}
+
 static void registerTypeResolver(TypeResolver* resolver)
 {
-    static bool initied = false;
-    if (!initied) {
-        assert(typeResolverMap.empty());
-        typeResolverMap.set_empty_key("");
-        typeResolverMap.set_deleted_key("?");
-        initied = true;
-        std::atexit(deinitTypeResolver);
-        TypeResolver::createValueTypeResolver<double>("double");
-        TypeResolver::createValueTypeResolver<float>("float");
-        TypeResolver::createValueTypeResolver<bool>("bool");
-        TypeResolver::createValueTypeResolver<int>("int");
-    }
-    assert(typeResolverMap.find(resolver->typeName()) == typeResolverMap.end());
     typeResolverMap[resolver->typeName()] = resolver;
 }
 
@@ -130,3 +125,25 @@ PyTypeObject* TypeResolver::pythonType()
 {
     return m_d->pyType;
 }
+
+TypeResolver::Type TypeResolver::getType(const char* name)
+{
+    std::string typeName(name);
+    int len = typeName.size() - 1;
+    if (len > 1) {
+        if (typeName[len] == '*')
+            typeName.erase(len, 1);
+
+        TypeResolver *resolver = TypeResolver::get(typeName.c_str());
+        if (resolver)
+            return TypeResolver::ValueType;
+
+        typeName += '*';
+        resolver = TypeResolver::get(typeName.c_str());
+        if (resolver)
+            return TypeResolver::ObjectType;
+    }
+
+    return TypeResolver::UnknownType;
+}
+
