@@ -44,6 +44,23 @@ QHash<QString, QString> ShibokenGenerator::m_pythonOperators = QHash<QString, QS
 QHash<QString, QString> ShibokenGenerator::m_formatUnits = QHash<QString, QString>();
 QHash<QString, QString> ShibokenGenerator::m_tpFuncs = QHash<QString, QString>();
 
+
+static QString resolveScopePrefix(const AbstractMetaClass* scope, const QString& value)
+{
+    if (!scope)
+        return QString();
+
+    QString name;
+    QStringList parts = scope->qualifiedCppName().split("::", QString::SkipEmptyParts);
+    for(int i = (parts.size() - 1) ; i >= 0; i--) {
+        if (!value.startsWith(parts[i] + "::"))
+            name = parts[i] + "::" + name;
+        else
+            name = "";
+    }
+
+    return name;
+}
 ShibokenGenerator::ShibokenGenerator() : Generator()
 {
     if (m_pythonPrimitiveTypeName.isEmpty())
@@ -153,12 +170,13 @@ void ShibokenGenerator::initPrimitiveTypesCorrespondences()
 }
 
 QString ShibokenGenerator::translateTypeForWrapperMethod(const AbstractMetaType* cType,
-                                                         const AbstractMetaClass* context) const
+                                                         const AbstractMetaClass* context,
+                                                         Options options) const
 {
     QString result;
 
     if (cType->isArray()) {
-        result = translateTypeForWrapperMethod(cType->arrayElementType(), context) + "[]";
+        result = translateTypeForWrapperMethod(cType->arrayElementType(), context, options) + "[]";
     } else {
 #ifdef AVOID_PROTECTED_HACK
         if (cType->isEnum()) {
@@ -168,7 +186,7 @@ QString ShibokenGenerator::translateTypeForWrapperMethod(const AbstractMetaType*
         }
         if (result.isEmpty())
 #endif
-            result = translateType(cType, context);
+            result = translateType(cType, context, options);
     }
 
     return result;
@@ -212,6 +230,23 @@ QString ShibokenGenerator::wrapperName(const AbstractMetaClass* metaClass)
     }
 }
 
+QString ShibokenGenerator::fullPythonFunctionName(const AbstractMetaFunction* func)
+{
+    QString funcName;
+    if (func->isOperatorOverload())
+        funcName = ShibokenGenerator::pythonOperatorFunctionName(func);
+    else
+       funcName = func->name();
+    if (func->ownerClass()) {
+        QString fullName = func->ownerClass()->fullName();
+        if (func->isConstructor())
+            funcName = fullName;
+        else
+            funcName.prepend(fullName + '.');
+    }
+    return funcName;
+}
+
 QString ShibokenGenerator::protectedEnumSurrogateName(const AbstractMetaEnum* metaEnum)
 {
     return metaEnum->fullName().replace(".", "_") + "_Surrogate";
@@ -261,6 +296,12 @@ QString ShibokenGenerator::cpythonGettersSettersDefinitionName(const AbstractMet
     return QString("%1_getsetlist").arg(cpythonBaseName(metaClass));
 }
 
+QString ShibokenGenerator::cpythonSetattroFunctionName(const AbstractMetaClass* metaClass)
+{
+    return QString("%1_setattro").arg(cpythonBaseName(metaClass));
+}
+
+
 QString ShibokenGenerator::cpythonGetattroFunctionName(const AbstractMetaClass* metaClass)
 {
     return QString("%1_getattro").arg(cpythonBaseName(metaClass));
@@ -283,6 +324,106 @@ static QString cpythonEnumFlagsName(QString moduleName, QString qualifiedCppName
     return result;
 }
 
+static QString searchForEnumScope(const AbstractMetaClass* metaClass, const QString& value)
+{
+    QString enumValueName = value.trimmed();
+
+    if (!metaClass)
+        return QString();
+
+    foreach (const AbstractMetaEnum* metaEnum, metaClass->enums()) {
+        foreach (const AbstractMetaEnumValue* enumValue, metaEnum->values()) {
+            if (enumValueName == enumValue->name())
+                return metaClass->qualifiedCppName();
+        }
+    }
+
+    return searchForEnumScope(metaClass->enclosingClass(), enumValueName);
+}
+
+/*
+ * This function uses some heuristics to find out the scope for a given
+ * argument default value. New situations may arise in the future and
+ * this method should be updated, do it with care.
+ */
+QString ShibokenGenerator::guessScopeForDefaultValue(const AbstractMetaFunction* func, const AbstractMetaArgument* arg)
+{
+    if (arg->defaultValueExpression().isEmpty())
+        return QString();
+
+    static QRegExp enumValueRegEx("^([A-Za-z_]\\w*)?$");
+    QString value = arg->defaultValueExpression();
+    QString prefix;
+    QString suffix;
+
+    if (arg->type()->isEnum()) {
+        const AbstractMetaEnum* metaEnum = findAbstractMetaEnum(arg->type());
+        prefix = resolveScopePrefix(metaEnum->enclosingClass(), value);
+    } else if (arg->type()->isFlags()) {
+        static QRegExp numberRegEx("^\\d+$"); // Numbers to flags
+        if (numberRegEx.exactMatch(value)) {
+            QString typeName = translateTypeForWrapperMethod(arg->type(), func->implementingClass());
+            if (arg->type()->isConstant())
+                typeName.remove(0, sizeof("const ") / sizeof(char) - 1);
+            if (arg->type()->isReference())
+                typeName.chop(1);
+            prefix = typeName + '(';
+            suffix = ')';
+        }
+
+        static QRegExp enumCombinationRegEx("^([A-Za-z_][\\w:]*)\\(([^,\\(\\)]*)\\)$"); // FlagName(EnumItem|EnumItem|...)
+        if (prefix.isEmpty() && enumCombinationRegEx.indexIn(value) != -1) {
+            QString flagName = enumCombinationRegEx.cap(1);
+            QStringList enumItems = enumCombinationRegEx.cap(2).split("|");
+            QString scope = searchForEnumScope(func->implementingClass(), enumItems.first());
+            if (!scope.isEmpty())
+                scope.append("::");
+
+            QStringList fixedEnumItems;
+            foreach (const QString& enumItem, enumItems)
+                fixedEnumItems << QString(scope + enumItem);
+
+            if (!fixedEnumItems.isEmpty()) {
+                prefix = flagName + '(';
+                value = fixedEnumItems.join("|");
+                suffix = ')';
+            }
+        }
+    } else if (arg->type()->typeEntry()->isValue()) {
+        const AbstractMetaClass* metaClass = classes().findClass(arg->type()->typeEntry());
+        if (enumValueRegEx.exactMatch(value))
+            prefix = resolveScopePrefix(metaClass, value);
+    } else if (arg->type()->isPrimitive() && arg->type()->name() == "int") {
+        if (enumValueRegEx.exactMatch(value) && func->implementingClass())
+            prefix = resolveScopePrefix(func->implementingClass(), value);
+    } else if(arg->type()->isPrimitive()) {
+        static QRegExp unknowArgumentRegEx("^(?:[A-Za-z_][\\w:]*\\()?([A-Za-z_]\\w*)(?:\\))?$"); // [PrimitiveType(] DESIREDNAME [)]
+        if (unknowArgumentRegEx.indexIn(value) != -1 && func->implementingClass()) {
+            foreach (const AbstractMetaField* field, func->implementingClass()->fields()) {
+                if (unknowArgumentRegEx.cap(1).trimmed() == field->name()) {
+                    QString fieldName = field->name();
+                    if (field->isStatic()) {
+                        prefix = resolveScopePrefix(func->implementingClass(), value);
+                        fieldName.prepend(prefix);
+                        prefix= "";
+                    } else {
+                        fieldName.prepend(CPP_SELF_VAR "->");
+                    }
+                    value.replace(unknowArgumentRegEx.cap(1), fieldName);
+                    break;
+                }
+            }
+        }
+    }
+
+    if (!prefix.isEmpty())
+        value.prepend(prefix);
+    if (!suffix.isEmpty())
+        value.append(suffix);
+
+    return value;
+}
+
 QString ShibokenGenerator::cpythonEnumName(const EnumTypeEntry* enumEntry)
 {
     return cpythonEnumFlagsName(enumEntry->targetLangPackage().replace(".", "_"), enumEntry->qualifiedCppName());
@@ -311,7 +452,7 @@ QString ShibokenGenerator::cpythonWrapperCPtr(const AbstractMetaType* metaType,
 QString ShibokenGenerator::cpythonWrapperCPtr(const TypeEntry* type, QString argName)
 {
     if (type->isValue() || type->isObject())
-        return baseConversionString(type->qualifiedCppName() + '*') + QString("toCpp((PyObject*)%1)").arg(argName);
+        return baseConversionString(type->qualifiedCppName() + '*') + "toCpp(" + argName + ')';
     return QString();
 }
 
@@ -357,24 +498,14 @@ void ShibokenGenerator::writeBaseConversion(QTextStream& s, const AbstractMetaTy
         if (ptype->basicAliasedTypeEntry())
             ptype = ptype->basicAliasedTypeEntry();
         typeName = ptype->name();
-    } else {
-        typeName = translateTypeForWrapperMethod(type, context);
+    } else { 
+        if (type->isObject() || (type->isValue() && !type->isReference()))
+            options |= Generator::ExcludeConst;
+	if (type->isContainer() )
+            options |= Generator::ExcludeReference | Generator::ExcludeConst;
+        typeName = translateTypeForWrapperMethod(type, context, options);
     }
 
-    const TypeEntry* tentry = type->typeEntry();
-
-    // If the type is an Object (and a pointer) remove its constness since it
-    // is already declared as const in the signature of the generated converter.
-    if (tentry->isObject() && typeName.startsWith("const "))
-        typeName.remove(0, sizeof("const ") / sizeof(char) - 1);
-
-    // Remove the constness, if any
-    if (typeName.startsWith("const ") && type->name() != "char")
-        typeName.remove(0, sizeof("const ") / sizeof(char) - 1);
-
-    if ((options & ExcludeReference || tentry->isPrimitive() || tentry->isContainer()) && typeName.endsWith('&'))
-        typeName.chop(1);
-
     s << baseConversionString(typeName);
 }
 
@@ -409,7 +540,7 @@ void ShibokenGenerator::writeToCppConversion(QTextStream& s, const AbstractMetaT
 QString ShibokenGenerator::getFormatUnitString(const AbstractMetaFunction* func, bool incRef) const
 {
     QString result;
-    const char objType = (incRef ? 'O' : 'S');
+    const char objType = (incRef ? 'O' : 'N');
     foreach (const AbstractMetaArgument* arg, func->arguments()) {
         if (func->argumentRemoved(arg->argumentIndex() + 1))
             continue;
@@ -420,6 +551,7 @@ QString ShibokenGenerator::getFormatUnitString(const AbstractMetaFunction* func,
             || arg->type()->isObject()
             || arg->type()->isValue()
             || arg->type()->isValuePointer()
+            || arg->type()->isNativePointer()
             || arg->type()->isEnum()
             || arg->type()->isFlags()
             || arg->type()->isContainer()
@@ -436,7 +568,12 @@ QString ShibokenGenerator::getFormatUnitString(const AbstractMetaFunction* func,
         } else if (isCString(arg->type())) {
             result += 'z';
         } else {
-            ReportHandler::warning("Unknown type used in ShibokenGenerator::getFormatUnitString!");
+            QString report;
+            QTextStream(&report) << "Method: " << func->ownerClass()->qualifiedCppName()
+                                 << "::" << func->signature() << " => Arg:"
+                                 << arg->name() << "index: " << arg->argumentIndex()
+                                 << " - cannot be handled properly. Use an inject-code to fix it!";
+            ReportHandler::warning(report);
             result += '?';
         }
     }
@@ -641,7 +778,7 @@ static QString checkFunctionName(QString baseName, bool genericNumberType, bool
            .arg((checkExact && !genericNumberType ? "Exact" : ""));
 }
 
-QString ShibokenGenerator::cpythonCheckFunction(const AbstractMetaType* metaType, bool genericNumberType, bool checkExact)
+QString ShibokenGenerator::cpythonCheckFunction(const AbstractMetaType* metaType, bool genericNumberType)
 {
     QString baseName = cpythonBaseName(metaType);
     if (metaType->typeEntry()->isCustom()) {
@@ -653,7 +790,7 @@ QString ShibokenGenerator::cpythonCheckFunction(const AbstractMetaType* metaType
         QTextStream s(&str);
         writeBaseConversion(s, metaType, 0);
         s.flush();
-        return str + "isConvertible";
+        return str + "checkType";
     }
 }
 
@@ -668,9 +805,27 @@ QString ShibokenGenerator::guessCPythonCheckFunction(const QString& type)
 {
     QString retval;
     AbstractMetaType* metaType = buildAbstractMetaTypeFromString(type);
-    if (metaType) {
+    if (metaType && (metaType->name() != type)) {
         retval = cpythonCheckFunction(metaType);
         delete metaType;
+        metaType = 0;
+    } else if (type == "PyTypeObject") {
+        retval = "PyType_Check";
+    } else {
+        retval = QString("%1_Check").arg(type);
+    }
+    if(metaType)
+        delete metaType;
+    return retval;
+}
+
+QString ShibokenGenerator::guessCPythonIsConvertible(const QString& type)
+{
+    QString retval;
+    AbstractMetaType* metaType = buildAbstractMetaTypeFromString(type);
+    if (metaType) {
+        retval = cpythonIsConvertibleFunction(metaType);
+        delete metaType;
     } else if (type == "PyTypeObject") {
         retval = "PyType_Check";
     } else {
@@ -679,8 +834,12 @@ QString ShibokenGenerator::guessCPythonCheckFunction(const QString& type)
     return retval;
 }
 
-QString ShibokenGenerator::cpythonIsConvertibleFunction(const TypeEntry* type)
+QString ShibokenGenerator::cpythonIsConvertibleFunction(const TypeEntry* type, bool genericNumberType, bool checkExact)
 {
+    if (checkExact)
+        return cpythonCheckFunction(type, genericNumberType, checkExact);
+    if (type->isCustom())
+        return guessCPythonIsConvertible(type->name());
     QString baseName;
     QTextStream s(&baseName);
     writeBaseConversion(s, type);
@@ -689,18 +848,21 @@ QString ShibokenGenerator::cpythonIsConvertibleFunction(const TypeEntry* type)
     return baseName;
 }
 
-QString ShibokenGenerator::cpythonIsConvertibleFunction(const AbstractMetaType* metaType)
+QString ShibokenGenerator::cpythonIsConvertibleFunction(const AbstractMetaType* metaType, bool genericNumberType)
 {
-    QString baseName;
-    QTextStream s(&baseName);
-    if (metaType->isValuePointer() || metaType->typeEntry()->isObject()) {
-        const AbstractMetaClass* context = classes().findClass(metaType->typeEntry()->name());
-        writeBaseConversion(s, metaType, context);
+
+    QString baseName = cpythonBaseName(metaType);
+    if (metaType->typeEntry()->isCustom()) {
+        return guessCPythonCheckFunction(metaType->typeEntry()->name());
+    } else if (isNumber(baseName)) {
+        return genericNumberType ? "PyNumber_Check" : baseName+"_Check";
     } else {
-        writeBaseConversion(s, metaType->typeEntry());
+        QString str;
+        QTextStream s(&str);
+        writeBaseConversion(s, metaType, 0);
+        s.flush();
+        return str + "isConvertible";
     }
-    s << "isConvertible";
-    s.flush();
     return baseName;
 }
 
@@ -720,7 +882,7 @@ QString ShibokenGenerator::argumentString(const AbstractMetaFunction *func,
 
     if (!(options & Generator::SkipName)) {
         arg += " ";
-        arg += argument->argumentName();
+        arg += argument->name();
     }
 
     QList<ReferenceCount> referenceCounts;
@@ -829,18 +991,7 @@ void ShibokenGenerator::writeArgumentNames(QTextStream &s,
 
         if (argCount > 0)
             s << ", ";
-
-        if ((options & Generator::BoxedPrimitive) &&
-            !arguments.at(j)->type()->isReference() &&
-            (arguments.at(j)->type()->isQObject() ||
-             arguments.at(j)->type()->isObject())) {
-            //s << "brian::wrapper_manager::instance()->retrieve( " << arguments.at(j)->argumentName() << " )";
-            // TODO: replace boost thing
-            Q_ASSERT(false);
-            s << "python::ptr( " << arguments.at(j)->argumentName() << " )";
-        } else {
-            s << arguments.at(j)->argumentName();
-        }
+        s << arguments.at(j)->name();
 
         if (((options & Generator::VirtualCall) == 0) &&
             (!func->conversionRule(TypeSystem::NativeCode, arguments.at(j)->argumentIndex() + 1).isEmpty() ||
@@ -926,9 +1077,7 @@ void ShibokenGenerator::writeCodeSnips(QTextStream& s,
                 argsRemoved++;
         }
         numArgs = func->arguments().size() - argsRemoved;
-
-        usePyArgs = getMinMaxArguments(func).second > 1 || func->isConstructor();
-
+        usePyArgs = getMinMaxArguments(func).second > 1;
     }
 
     foreach (CodeSnip snip, codeSnips) {
@@ -1055,7 +1204,7 @@ void ShibokenGenerator::writeCodeSnips(QTextStream& s,
                         }
                     }
                 } else {
-                    argReplacement = arg->argumentName();
+                    argReplacement = arg->name();
                 }
                 code.replace("%" + QString::number(i+1), argReplacement);
             }
@@ -1082,7 +1231,7 @@ void ShibokenGenerator::writeCodeSnips(QTextStream& s,
                     }
                     argumentNames << argName;
                 } else {
-                    argumentNames << arg->argumentName();
+                    argumentNames << arg->name();
                 }
             }
             code.replace("%ARGUMENT_NAMES", argumentNames.join(", "));
@@ -1210,30 +1359,6 @@ bool ShibokenGenerator::hasMultipleInheritanceInAncestry(const AbstractMetaClass
     return hasMultipleInheritanceInAncestry(metaClass->baseClass());
 }
 
-bool ShibokenGenerator::needsReferenceCountControl(const AbstractMetaClass* metaClass)
-{
-    if (!metaClass->fields().isEmpty() || hasMethodsWithReferenceCountModifications(metaClass))
-        return true;
-    foreach (const AbstractMetaClass* parent, getBaseClasses(metaClass)) {
-        if (needsReferenceCountControl(parent))
-            return true;
-    }
-    return false;
-}
-
-bool ShibokenGenerator::hasMethodsWithReferenceCountModifications(const AbstractMetaClass* metaClass)
-{
-    foreach (const AbstractMetaFunction* func, metaClass->functions()) {
-        foreach (FunctionModification func_mod, func->modifications()) {
-            foreach (ArgumentModification arg_mod, func_mod.argument_mods) {
-                if (!arg_mod.referenceCounts.isEmpty())
-                    return true;
-            }
-        }
-    }
-    return false;
-}
-
 bool ShibokenGenerator::classNeedsGetattroFunction(const AbstractMetaClass* metaClass)
 {
     if (!metaClass)
