@@ -43,7 +43,7 @@ public:
     ShibokenGenerator();
 
     QString translateTypeForWrapperMethod(const AbstractMetaType* cType,
-                                          const AbstractMetaClass* context) const;
+                                          const AbstractMetaClass* context, Options opt = NoOption) const;
 
     /**
     *   Returns a map with all functions grouped, the function name is used as key.
@@ -178,12 +178,6 @@ public:
     /// Returns true if there are cases of multiple inheritance in any of its ancestors.
     bool hasMultipleInheritanceInAncestry(const AbstractMetaClass* metaClass);
 
-    /// Returns true if the class needs reference counting control.
-    bool needsReferenceCountControl(const AbstractMetaClass* metaClass);
-
-    /// Returns true if the class has any method that modifies the reference counting of any of its arguments.
-    bool hasMethodsWithReferenceCountModifications(const AbstractMetaClass* metaClass);
-
     /// Returns true if the class needs to have a getattro function.
     bool classNeedsGetattroFunction(const AbstractMetaClass* metaClass);
 
@@ -213,6 +207,7 @@ public:
 
     static QString wrapperName(const AbstractMetaClass* metaClass);
 
+    static QString fullPythonFunctionName(const AbstractMetaFunction* func);
     static QString protectedEnumSurrogateName(const AbstractMetaEnum* metaEnum);
     static QString protectedFieldGetterName(const AbstractMetaField* field);
     static QString protectedFieldSetterName(const AbstractMetaField* field);
@@ -247,23 +242,29 @@ public:
     QString cpythonTypeName(const TypeEntry* type);
     QString cpythonTypeNameExt(const TypeEntry* type);
     QString cpythonCheckFunction(const TypeEntry* type, bool genericNumberType = false, bool checkExact = false);
-    QString cpythonCheckFunction(const AbstractMetaType* metaType, bool genericNumberType = false, bool checkExact = false);
+    QString cpythonCheckFunction(const AbstractMetaType* metaType, bool genericNumberType = false);
     QString guessCPythonCheckFunction(const QString& type);
-    QString cpythonIsConvertibleFunction(const TypeEntry* type);
-    QString cpythonIsConvertibleFunction(const AbstractMetaType* metaType);
-    QString cpythonIsConvertibleFunction(const AbstractMetaArgument* metaArg) {
-        return cpythonIsConvertibleFunction(metaArg->type());
+    QString cpythonIsConvertibleFunction(const TypeEntry* type, bool genericNumberType = false, bool checkExact = false);
+    QString cpythonIsConvertibleFunction(const AbstractMetaType* metaType, bool genericNumberType = false);
+    QString cpythonIsConvertibleFunction(const AbstractMetaArgument* metaArg, bool genericNumberType = false)
+    {
+        return cpythonIsConvertibleFunction(metaArg->type(), genericNumberType);
     }
+    QString guessCPythonIsConvertible(const QString& type);
     QString cpythonFunctionName(const AbstractMetaFunction* func);
     QString cpythonMethodDefinitionName(const AbstractMetaFunction* func);
     QString cpythonGettersSettersDefinitionName(const AbstractMetaClass* metaClass);
     QString cpythonGetattroFunctionName(const AbstractMetaClass* metaClass);
+    QString cpythonSetattroFunctionName(const AbstractMetaClass* metaClass);
     QString cpythonGetterFunctionName(const AbstractMetaField* metaField);
     QString cpythonSetterFunctionName(const AbstractMetaField* metaField);
     QString cpythonWrapperCPtr(const AbstractMetaClass* metaClass, QString argName = "self");
     QString cpythonWrapperCPtr(const AbstractMetaType* metaType, QString argName);
     QString cpythonWrapperCPtr(const TypeEntry* type, QString argName);
 
+    /// Guesses the scope to where belongs an argument's default value.
+    QString guessScopeForDefaultValue(const AbstractMetaFunction* func, const AbstractMetaArgument* arg);
+
     QString cpythonEnumName(const EnumTypeEntry* enumEntry);
     QString cpythonEnumName(const AbstractMetaEnum* metaEnum)
     {
